<?php
// $Id$

/**
 * @file
 * Admin functions
 */

define('BOCU_FILES_PATH',         file_directory_path());
define('BOCU_IMG_MAXWIDTH',       0);
define('BOCU_IMG_MAXHEIGHT',      0);
define('BOCU_IMG_THUMB_TEMPLATE', '<a href="!filepath" target="_blank"><img src="!thumbpath" alt="" /></a>');
define('BOCU_IMAGECACHE_PRESET',  '');


/**
 * CRUD operation: Добавляет шаблон в таблицу bocu_templates
 */
function bocu_template_add($template)
{
    if (isset($template['roles']) && is_array($template['roles']))
    {
        $template['roles'] = implode(',', $template['roles']);
    }
    if (isset($template['rename_file']) && !is_bool($template['rename_file']))
    {
        $template['rename_file'] = (int)(bool)$template['rename_file'];
    }
    
    return drupal_write_record('bocu_templates', $template);
}


/**
 * CRUD operation: Обновляет шаблон в таблице bocu_templates
 */
function bocu_template_save($template)
{
    if (isset($template['roles']) && is_array($template['roles']))
    {
        $template['roles'] = implode(',', $template['roles']);
    }
    
    return drupal_write_record('bocu_templates', $template, 'tid');
}


/**
 * CRUD operation: Удаление шаблона
 */
function bocu_template_delete($tid)
{
    db_query('DELETE FROM {bocu_templates} WHERE tid = %d', $tid);
}


/**
 * CRUD operation: Список шаблонов
 */
function bocu_templates_list($onlyUseRoles = false)
{
    $templates = db_query("
        SELECT * FROM {bocu_templates}
        WHERE 1 = 1 " . ($onlyUseRoles ? "AND roles <> ''" : "") . "
        ORDER BY tid
    ");
    $templatesArray = array();
    
    while ($template = db_fetch_array($templates))
    {
        $template['mask_string']  = $template['mask'];
        $template['roles_string'] = $template['roles'];
        
        $template['mask']  = explode(',', $template['mask']);
        $template['roles'] = explode(',', $template['roles']);
        
        $templatesArray[] = $template;
    }
    
    return $templatesArray;
}


/**
 * Форма с настройками модуля
 */
function bocu_form_settings()
{
    $imagecachePresetOptions = array('' => 'нет');
    $buttonimgOptions = array(BOCU_BUTTONIMG => 'upload-20.png');
    $icons = array_merge(
        file_scan_directory(drupal_get_path('module', 'bocu') . '/static', '\.png$', array('.', '..', 'CVS'), 0, false),
        file_scan_directory(drupal_get_path('module', 'bueditor') . '/icons', '\.(jpe?g|gif|png)$')
    );
    
    if (module_exists('imagecache'))
    {
        $presets = imagecache_presets();
        
        foreach ($presets as $preset)
        {
            $imagecachePresetOptions[$preset['presetname']] = $preset['presetname'];
        }
    }
    
    foreach ($icons as $icon)
    {
        $buttonimgOptions[$icon->filename] = substr($icon->filename, 18);
    }
    
    $form['bocu_button_settings'] = array(
        '#type' => 'fieldset',
        '#title' => 'Настройки кнопки',
        '#collapsible' => true,
        '#collapsed' => false,
    );
    
    $form['bocu_button_settings']['bocu_buttonimg'] = array(
        '#type' => 'select',
        '#title' => 'Иконка',
        '#description' => 'Иконка, которая будет отображаться в редакторе.',
        '#default_value' => variable_get('bocu_buttonimg', BOCU_BUTTONIMG),
        '#options' => $buttonimgOptions,
        '#required' => true,
        '#attributes' => array('onchange' => '
            $(this).next("img").remove();
            $("<img />")
                .attr("src", Drupal.settings.basePath + this.value)
                .attr("alt", "")
                .css({"vertical-align":"middle", "margin":"0 0 2px 5px", "border":"1px solid #CCCCCC"})
                .insertAfter(this);
        '),
    );
    
    $form['bocu_button_settings']['bocu_buttonimg_script'] = array(
        '#value' => '<script type="text/javascript">$("#edit-bocu-buttonimg").change();</script>',
    );
    
    $form['bocu_button_settings']['bocu_forms'] = array(
        '#type' => 'checkboxes',
        '#title' => 'Показывать кнопку в формах',
        '#description' => 'Отметьте формы, в которых должна появится кнопка модуля.',
        '#options' => array('node_form' => t('Node form'), 'comment_form' => t('Comment form')),
        '#default_value' => variable_get('bocu_forms', array(BOCU_FORMS)),
    );
    
    $form['bocu_img_settings'] = array(
        '#type' => 'fieldset',
        '#title' => 'Настройки для загружаемых изображений',
        '#collapsible' => true,
        '#collapsed' => false,
    );
    
    $form['bocu_img_settings']['bocu_maxsize'] = array(
        '#type' => 'markup',
        '#theme' => 'inline_fields',
        '#description' => 'Ширина и высота, в пикселях. Если размеры загруженного изображение больше введённых значений, то для него (изображения) будет создан эскиз, и в редактор вставится шаблон эскиза.',
        '#title' => 'Максимальные размеры картинки',
        '#fields_separator' => ' × ',
    );
    
    $form['bocu_img_settings']['bocu_maxsize']['bocu_img_maxwidth'] = array(
        '#type' => 'textfield',
        '#title' => 'Ширина',
        '#default_value' => variable_get('bocu_img_maxwidth', BOCU_IMG_MAXWIDTH),
        '#size' => 4,
    );
    
    $form['bocu_img_settings']['bocu_maxsize']['bocu_img_maxheight'] = array(
        '#type' => 'textfield',
        '#title' => 'Высота',
        '#default_value' => variable_get('bocu_img_maxheight', BOCU_IMG_MAXHEIGHT),
        '#size' => 4,
        '#field_suffix' => 'px',
    );
    
    $form['bocu_img_settings']['bocu_img_thumb_template'] = array(
        '#type' => 'textfield',
        '#title' => 'Шаблон для вставки эскиза',
        '#default_value' => variable_get('bocu_img_thumb_template', BOCU_IMG_THUMB_TEMPLATE),
        '#maxlength' => null,
        '#size' => 100,
        '#required' => true,
    );
    
    $form['bocu_img_settings']['bocu_img_template_help'] = array(
        '#type' => 'fieldset',
        '#title' => 'Описание переменных',
        '#collapsible' => true,
        '#collapsed' => true,
    );
    
    $form['bocu_img_settings']['bocu_img_template_help']['text'] = array(
        '#value' => '
            <div class="tips">
                !filepath — полный путь к изображению<br />
                !filename — имя файла после загрузки<br />
                !originalname — оригинальное имя файла<br />
                !filesize — размер файла<br />
                !imgwidth — ширина изображения<br />
                !imgheight — высота изображения<br />
                !thumbpath — путь к эскизу<br />
                !thumbwidth — ширина эскиза (доступно только при отключённом профиле ImageCache)<br />
                !thumbheight — высота эскиза (доступно только при отключённом профиле ImageCache)<br />
            </div>
        ',
    );
    
    $form['bocu_img_settings']['bocu_imagecache_preset'] = array(
        '#type' => 'select',
        '#title' => 'Профиль ImageCache',
        '#description' => 'Набор правил, которые будут применены для загруженных изображений.',
        '#default_value' => variable_get('bocu_imagecache_preset', BOCU_IMAGECACHE_PRESET),
        '#options' => $imagecachePresetOptions,
        '#disabled' => !module_exists('imagecache'),
    );
    
    $form['bocu_img_settings']['bocu_quality'] = array(
        '#type' => 'markup',
        '#value' => l('Настроить качество обработки изображений', 'admin/settings/image-toolkit', array('attributes' => array('target' => '_blank'))),
    );
    
    return system_settings_form($form);
}


/**
 * Валидация формы настроек
 */
function bocu_form_settings_validate($form, &$form_state)
{
    if (!file_exists($form_state['values']['bocu_buttonimg']))
    {
        form_set_error('bocu_buttonimg', 'Файла не существует');
    }
    else
    {
        $imageInfo = image_get_info($form_state['values']['bocu_buttonimg']);
        
        if ($imageInfo)
        {
            variable_set('bocu_buttonimg_width',  $imageInfo['width']);
            variable_set('bocu_buttonimg_height', $imageInfo['height']);
        }
        else
        {
            form_set_error('bocu_buttonimg', 'Файл не является картинкой');
        }
    }

    if (!is_numeric($form_state['values']['bocu_img_maxwidth']))
    {
        form_set_error('bocu_img_maxwidth', 'Введите число');
    }
    if (!is_numeric($form_state['values']['bocu_img_maxheight']))
    {
        form_set_error('bocu_img_maxheight', 'Введите число');
    }
}


/**
 * Theme функция вывода полей в один ряд
 */
function theme_inline_fields($form)
{
    $fields = array();
    
    foreach ($form as $key => $element)
    {
        // ищем дочерние поля
        if (substr($key, 0, 1) != '#')
        {
            $size = empty($element['#size']) ? '' : ' size="'. $element['#size'] .'"';
            
            _form_set_class($element, array());
            
            $fields[] = '<input type="text" name="' . $element['#name']  . '" id="'. $element['#id']  . '"' . $size . ' value="' . check_plain($element['#value'])  . '"' . drupal_attributes($element['#attributes']) . ' />';
        }
    }
    
    return theme('form_element', $form, implode($form['#fields_separator'], $fields));
}


/**
 * Форма добавления шаблона
 */
function bocu_form_template($form_state, $template = null)
{
    $isAddForm = false;
    
    if (!$template)
    {
        $template = array(
            'name' => '',
            'mask' => '',
            'path' => '',
            'template' => '<a href="!filepath">!filename</a>',
            'template_select' => '<a href="!filepath">!text</a>',
            'roles' => array(),
            'rename_file' => 1,
        );
        
        $isAddForm = true;
    }
    
    $form['name'] = array(
        '#type' => 'textfield',
        '#title' => 'Название шаблона',
        '#description' => 'Будет отображаться в ' . l('списке шаблонов', 'admin/settings/bueditor/bocu/templates_list', array('alias' => true)) . '.',
        '#default_value' => $template['name'],
        '#size' => 50,
        '#required' => true,
    );
    
    $form['mask'] = array(
        '#type' => 'textfield',
        '#title' => 'Расширения файлов',
        '#description' => 'Список расширений файлов, через запятую, которые нужно обрабатывать в рамках этого шаблона. Например: jpg,gif,png',
        '#default_value' => $template['mask'],
        '#size' => 50,
        '#required' => true,
    );
    
    $form['path'] = array(
        '#type' => 'textfield',
        '#title' => 'Директория для загрузки',
        '#description' => 'Имя директории, относительно ' .  BOCU_FILES_PATH . ', в которую будут загружаться файлы. Без начального и завершающего слэша. Если директория отсутствует, то она будет создана.',
        '#default_value' => $template['path'],
        '#field_prefix' => BOCU_FILES_PATH . '/',
        '#size' => 15,
        '#required' => true,
    );
    
    $form['template'] = array(
        '#type' => 'textfield',
        '#title' => 'Шаблон для вставки в редактор',
        '#default_value' => $template['template'],
        '#maxlength' => null,
        '#size' => 100,
        '#required' => true,
    );
    
    $form['template_select'] = array(
        '#type' => 'textfield',
        '#title' => 'Шаблон для замены выделенного текста',
        '#default_value' => $template['template_select'],
        '#maxlength' => null,
        '#size' => 100,
        '#required' => true,
    );
    
    $form['help'] = array(
        '#type' => 'fieldset',
        '#title' => 'Описание переменных',
        '#collapsible' => true,
        '#collapsed' => true,
    );
    
    $form['help']['text'] = array(
        '#value' => '
            <div class="tips">
                !filepath — полный путь к файлу<br />
                !filename — имя файла после загрузки<br />
                !originalname — оригинальное имя файла<br />
                !filesize — размер файла<br />
                !imgwidth — ширина изображения<br />
                !imgheight — высота изображения<br />
                !text — выделенный текст<br />
            </div>
        ',
    );
    
    $form['roles'] = array(
        '#type' => 'checkboxes',
        '#title' => 'Роли',
        '#description' => 'Отметьте роли, которые смогут загружать файлы с указанными выше расширениями.',
        '#options' => array('admin' => t('user') .  ' #1') + user_roles(),
        '#default_value' => $template['roles'],
    );
    
    $form['rename_file'] = array(
        '#type' => 'radios',
        '#title' => 'Давать загружаемым файлам уникальные имена',
        '#description' => 'Выберите <b>Да</b>, если хотите чтобы загружаемым файлам давались уникальные имена (дата и время загрузки).',
        '#options' => array(1 => 'Да', 0 => 'Нет'),
        '#default_value' => $template['rename_file'],
    );
    
    $form['submit'] = array(
        '#type' => 'submit',
        '#value' => $isAddForm ? t('Add template') : t('Save template'),
        '#submit' => $isAddForm ? array('bocu_form_template_submit_add') : array('bocu_form_template_submit_save'),
    );
    
    if (!$isAddForm)
    {
        $form['tid'] = array(
            '#type' => 'value',
            '#value' => $template['tid'],
        );
    }
    else
    {
        $form['#redirect'] = 'admin/settings/bueditor/bocu/templates_list';
    }
    
    return $form;
}


/**
 * Валидация формы добавления шаблона
 */
function bocu_form_template_validate($form, &$form_state)
{
    $fullPath = BOCU_FILES_PATH . '/' . $form_state['values']['path'];
    file_check_directory($fullPath, FILE_CREATE_DIRECTORY, 'path');
}


/**
 * Добавление шаблона через форму
 */
function bocu_form_template_submit_add($form, &$form_state)
{
    $values = $form_state['values'];
    
    bocu_template_add(array(
        'name'            => $values['name'],
        'mask'            => $values['mask'],
        'path'            => $values['path'],
        'template'        => $values['template'],
        'template_select' => $values['template_select'],
        'rename_file'     => $values['rename_file'],
        'roles'           => _bocu_get_checked_roles($values['roles']),
    ));
    
    drupal_set_message('Шаблон добавлен');
}


/**
 * Добавление шаблона через форму
 */
function bocu_form_template_submit_save($form, &$form_state)
{
    $values = $form_state['values'];
    
    bocu_template_save(array(
        'tid'             => $values['tid'],
        'name'            => $values['name'],
        'mask'            => $values['mask'],
        'path'            => $values['path'],
        'template'        => $values['template'],
        'template_select' => $values['template_select'],
        'rename_file'     => $values['rename_file'],
        'roles'           => _bocu_get_checked_roles($values['roles']),
    ));
    
    drupal_set_message('Шаблон обновлён');
}


/**
 * Форма подтверждения удаления шаблона
 */
function bocu_form_delete_confirm(&$form_state, $tid)
{
    $form['tid'] = array(
        '#type' => 'value',
        '#value' => $tid,
    );
 
    return confirm_form(
        $form,
        t('Are you sure you want to delete %title?', array('%title' => '')),
        'admin/settings/bueditor/bocu/templates_list',
        t('This action cannot be undone.'),
        t('Delete'),
        t('Cancel')
    );
}


/**
 * Удаление шаблона
 */
function bocu_form_delete_confirm_submit($form, &$form_state)
{
    if ($form_state['values']['confirm'])
    {
        bocu_template_delete($form_state['values']['tid']);            
        drupal_set_message('Шаблон удалён');
    }
 
    drupal_goto('admin/settings/bueditor/bocu/templates_list');
}


/**
 * Menu callback для url "admin/settings/bueditor/bocu/templates"
 * Выводит список шаблонов и ссылки на редактирование
 */
function bocu_templates_show_list()
{
    $tableHeader = array(t('Name'), t('Mask'), t('Actions'));
    $tableData = array();
    $templates = bocu_templates_list();
    
    foreach ($templates as $template)
    {
        $actions  = l(t('Edit'),   'admin/settings/bueditor/bocu/edit_template/' . $template['tid'],   array('alias' => true)) . ' | ';
        $actions .= l(t('Delete'), 'admin/settings/bueditor/bocu/delete_template/' . $template['tid'], array('alias' => true));
        
        $tableData[] = array(
            $template['name'],
            $template['mask_string'],
            $actions,
        );
    }
    
    return theme('table', $tableHeader, $tableData);
}


/**
 * Menu callback для адреса "bocu/upload"
 * Загрузка файла на сервер
 */
function bocu_upload($index)
{
    _bocu_auth();
    
    $inputName = 'bocu' . $index;
    $fileExt = isset($_FILES['files']['name'][$inputName]) ? _bocu_get_file_ext($_FILES['files']['name'][$inputName]) : '';
    $isImage = in_array($fileExt, array('jpg', 'jpeg', 'gif', 'png'));
    $isThumbLink = (isset($_POST['isThumbLink']) && $_POST['isThumbLink'] == 1);
    $textSelected = (isset($_POST['textSelected']) && $_POST['textSelected'] == 1);
    $template = _bocu_get_right_template($fileExt);
    $validators = array();
    $return = array();
    
    if (!$template)
    {
        $return['status'] = false;
        $return['data'] = 'Вам не разрешено загружать файлы этого типа';
    }
    else
    {
        if ($isImage)
        {
            $validators['file_validate_is_image'] = array();
        }
    
        $file = file_save_upload($inputName, $validators, _bocu_template_path($template));

        // Файл загружен успешно
        if ($file)
        {
            @chmod($file->filepath, 0644);
            
            // Если в шаблоне включена опция переименовывания файла
            if ($template['rename_file'])
            {
                _bocu_rename_file($file, $fileExt, $template, $isThumbLink);
            }
            
            $return['status'] = true;
            $return['data'] = $isImage ?
                _bocu_get_code_for_image($template, $file, $textSelected, $isThumbLink) :
                _bocu_get_code_for_file($template, $file, $textSelected);
        }
        // При загрузке возникли ошибки
        else
        {
            $messages = drupal_get_messages(null, true);
            
            foreach ($messages['error'] as $message)
            {
                $return['data'] .= strip_tags($message) . "\n\n";
            }
            
            $return['status'] = false;
            $return['data'] = isset($return['data']) ? trim($return['data']) : 'Ошибка при загрузке';
        }
    }
    
    echo drupal_json($return);
}


/**
 * Возвращает подходящий шаблон
 */
function _bocu_get_right_template($fileExt)
{
    global $user;
    
    $templates = bocu_templates_list(true);
    
    foreach ($templates as $template)
    {
        if (_bocu_check_role($template['roles']) && _bocu_check_ext($template['mask'], $fileExt))
        {
            return $template;
        }
    }
    
    return false;
}


/**
 * Проверка роли
 */
function _bocu_check_role($roles)
{
    global $user;
    
    // в $intersect будет массив ролей которые есть в обоих массивах - $template['roles'] и array_keys($user->roles)
    $intersect = array_intersect($roles, array_keys($user->roles));
    
    return (count($intersect) > 0 || in_array('admin', $roles) && $user->uid == 1);
}


/**
 * Проверка расширения файла
 */
function _bocu_check_ext($templateMask, $fileExt)
{
    return ($fileExt && in_array($fileExt, $templateMask));
}


/**
 * Авторизация по данным из $_POST
 */
function _bocu_auth()
{
    $phpsessid = isset($_POST['phpsessid']) ? $_POST['phpsessid'] : false;
    
    if ($phpsessid && session_id() != $phpsessid)
    {
        session_destroy();
        session_id($phpsessid);
        $_COOKIE[session_name()] = $phpsessid;
        _drupal_bootstrap(DRUPAL_BOOTSTRAP_SESSION);
    }
}


/**
 * Возвращает список выбранных ролей
 */
function _bocu_get_checked_roles($roles)
{
    $checkedRoles = array();
    
    foreach ($roles as $roleId => $checked)
    {
        if ($checked)
        {
            $checkedRoles[] = $roleId;
        }
    }
    
    return $checkedRoles;
}


/**
 * Возвращает код для загруженной картинки
 */
function _bocu_get_code_for_image($template, $file, $textSelected, $isThumbLink)
{
    global $base_path, $base_url;
    
    $presetname    = variable_get('bocu_imagecache_preset',  BOCU_IMAGECACHE_PRESET);
    $maxWidth      = variable_get('bocu_img_maxwidth',       BOCU_IMG_MAXWIDTH);
    $maxHeight     = variable_get('bocu_img_maxheight',      BOCU_IMG_MAXHEIGHT);
    $templateThumb = variable_get('bocu_img_thumb_template', BOCU_IMG_THUMB_TEMPLATE);
    $imageInfo     = image_get_info($file->filepath);

    // Если в настройках НЕ выбран пресет ImageCache, или в редакторе есть выделенный текст, или загружается "ссылка-эскиз"
    if (!$presetname || $textSelected || $isThumbLink)
    {
        // Если есть выделенный текст
        if ($textSelected)
        {
            return strtr($template['template_select'], _bocu_get_placeholders($file, $imageInfo));
        }
        // Если это "ссылка-эскиз"
        else if ($isThumbLink)
        {
            if (preg_match('#-thumb\.[a-z]{3,4}#', $file->originalname))
            {
                $_SESSION['bocu']['thumb'] = array('fileInfo' => $file, 'imageInfo' => $imageInfo);
            }
            else
            {
                $_SESSION['bocu']['image'] = array('fileInfo' => $file, 'imageInfo' => $imageInfo);
            }
            
            if (isset($_SESSION['bocu']['thumb'], $_SESSION['bocu']['image']))
            {
                $return = strtr($templateThumb, _bocu_get_placeholders(
                    $_SESSION['bocu']['image']['fileInfo'],
                    $_SESSION['bocu']['image']['imageInfo'],
                    $_SESSION['bocu']['thumb']['fileInfo'],
                    $_SESSION['bocu']['thumb']['imageInfo']
                ));
                
                unset($_SESSION['bocu']['thumb']);
                unset($_SESSION['bocu']['image']);
                
                return $return;
            }
            
            return '';
        }
        // Если изображение больше заданых размеровз
        else if (($maxWidth > 0 || $maxHeight > 0) && ($imageInfo['width'] > $maxWidth || $imageInfo['height'] > $maxHeight))
        {
            $thumb['filepath'] = preg_replace('#(.+)(\.\w{3,4})$#', '$1-thumb$2', $file->filepath);
        
            if (image_scale($file->filepath, $thumb['filepath'], $maxWidth, $maxHeight))
            {
                @chmod($thumb['filepath'], 0644);
                
                $thumb['uid']       = $file->uid;
                $thumb['filename']  = basename($thumb['filepath']);
                $thumb['filemime']  = $file->filemime;
                $thumb['filesize']  = filesize($thumb['filepath']);
                $thumb['timestamp'] = $file->timestamp;
                
                drupal_write_record('files', $thumb);
                
                $thumbInfo = image_get_info($thumb['filepath']);
                
                return strtr($templateThumb, _bocu_get_placeholders($file, $imageInfo, $thumb, $thumbInfo));
            }
        }
        // Если в редакторе нет выделенного текста, если это не "ссылка-эскиз" и размеры картинки меньше $maxWidth и $maxHeight
        else
        {
            return strtr($template['template'], _bocu_get_placeholders($file, $imageInfo));
        }
    }
    // Иначе возвращаем специально сформированный url картинки
    else 
    {
        $imagecacheUrl = str_replace($base_url . '/', '', imagecache_create_url($presetname, $file->filepath));
        
        // Если изображение слишком большое
        if ($imageInfo && ($imageInfo['width'] > $maxWidth || $imageInfo['height'] > $maxHeight))
        {
            return strtr($templateThumb, array(
                '!thumbpath' => $base_path . $imagecacheUrl,
            ) + _bocu_get_placeholders($file, $imageInfo));
        }
        else
        {
            return strtr($templateText, array(
                '!filepath' => $base_path . $imagecacheUrl,
            ) + _bocu_get_placeholders($file));
        }
    }
}


/**
 * Возвращает код для загруженного файла
 */
function _bocu_get_code_for_file($template, $file, $textSelected)
{
    global $base_path;

    $templateText = $textSelected ? $template['template_select'] : $template['template'];
    
    return strtr($templateText, _bocu_get_placeholders($file));
}


/**
 * Возвращает расширение файла
 */
function _bocu_get_file_ext($filename)
{
    if (preg_match('#\.([a-z0-9]+)$#i', $filename, $matches))
    {
        return $matches[1];
    }
    
    return '';
}


/**
 * Возвращает директорию для загрузки у шаблона $template
 */
function _bocu_template_path($template)
{
    return BOCU_FILES_PATH . '/' . $template['path'];
}


/**
 * Переименовывает файл
 */
function _bocu_rename_file(&$file, $fileExt, $template, $isThumbLink)
{
    $newFilename = date('YmdHis');
    
    if ($isThumbLink)
    {
        // если заливается картинка и эскиз уже залит
        if (isset($_SESSION['bocu']['thumb']))
        {
            preg_match('#^(.+)-thumb\.(jpe?g|gif|png)$#i', $_SESSION['bocu']['thumb']['fileInfo']->filename, $matches);
            $newFilename = $matches[1];
        }
        // если заливается эскиз и картинка уже залита
        else if (isset($_SESSION['bocu']['image']))
        {
            preg_match('#^(.+)\.(jpe?g|gif|png)$#i', $_SESSION['bocu']['image']['fileInfo']->filename, $matches);
            $newFilename = $matches[1] . '-thumb';
        }
        // если заливается эскиз и картинка ещё не залита
        else if (preg_match('#^.+-thumb\.' . $fileExt . '$#', $file->filename))
        {
            $newFilename .= '-thumb';
        }
    }
    
    $newFilename .= '.' . $fileExt;
    $newFilepath = _bocu_template_path($template) . '/' . $newFilename;
    
    if (file_move($file->filepath, $newFilepath))
    {
        db_query("
            UPDATE {files} SET filename = '%s', filepath = '%s'
            WHERE fid = %d
        ", $newFilename, $newFilepath, $file->fid);
        
        $file->filepath = $newFilepath;
        $file->originalname = $file->filename;
        $file->filename = $newFilename;
    }
}


/**
 * Возвращает информацию о файле
 */
function _bocu_get_placeholders($fileInfo, $imageInfo = null, $thumbFileInfo = null, $thumbImageInfo = null)
{
    $placeholders = array(
        '!filepath'     => $GLOBALS['base_path'] . $fileInfo->filepath,
        '!filename'     => $fileInfo->filename,
        '!originalname' => $fileInfo->originalname,
        '!filesize'     => format_size(filesize($fileInfo->filepath)),
    );
    
    if ($imageInfo)
    {
        $imageInfo = (object)$imageInfo;
        $placeholders['!imgwidth']  = $imageInfo->width;
        $placeholders['!imgheight'] = $imageInfo->height;
    }
    
    if ($thumbFileInfo)
    {
        $thumbFileInfo = (object)$thumbFileInfo;
        $placeholders['!thumbpath'] = $GLOBALS['base_path'] . $thumbFileInfo->filepath;
    }
    
    if ($thumbImageInfo)
    {
        $thumbImageInfo = (object)$thumbImageInfo;
        $placeholders['!thumbwidth']  = $thumbImageInfo->width;
        $placeholders['!thumbheight'] = $thumbImageInfo->height;
    }
    
    return $placeholders;
}
